/*
 * named.c - Databases of named values
 *
 * Written 2001,2002 by Werner Almesberger
 * Copyright 2001 Network Robots
 * Copyright 2002 Bivio Networks
 */


#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "error.h"
#include "u128.h"
#include "named.h"


const static struct ether_proto {
    const char *in_name;	/* protocol name for name->number lookup */
    uint16_t number;		/* protocol number, in host byte order */
    const char *out_name;	/* protocol name for number->name lookup;
				   NULL if translation is not supported */
} ether_proto_db[] = {
    { "ip",	ETH_P_IP,	"ip" },
    { "ipv4",	ETH_P_IP,	"ip" },
    { "ipv6",	ETH_P_IPV6,	"ipv6" },
    { NULL,	0 }
};


uint16_t ether_proto(const char *name)
{
    const struct ether_proto *ep;

    for (ep = ether_proto_db; ep->in_name; ep++)
	if (!strcmp(ep->in_name,name)) return ep->number;
    yyerrorf("unknown ether protocol \"%s\"",name);
}


const char *ether_proto_print(uint16_t proto)
{
    static char buf[7]; /* 0xffff\0 */
    const struct ether_proto *ep;

    for (ep = ether_proto_db; ep->in_name; ep++)
	if (ep->number == proto) {
	    if (ep->out_name) return ep->out_name;
	    break;
	}
    sprintf(buf,"0x%x",(unsigned) proto);
    return buf;
}


uint8_t ip_proto(const char *name)
{
    struct protoent *pe;

    pe = getprotobyname(name);
    if (!pe) yyerrorf("unknown IP protocol \"%s\"",name);
    return pe->p_proto;
}


/* @@@ gross hack ... should know L4 protocol too */
uint16_t ip_port(const char *name)
{
    struct servent *se;

    se = getservbyname(name,"tcp");
    if (!se) se = getservbyname(name,"udp");
    if (!se) yyerrorf("unknown service/port \"%s\"",name);
    return ntohs(se->s_port);
}


uint32_t ipv4_host(const char *name)
{
    struct hostent *he;
    uint32_t addr;

    addr = inet_addr(name);
    if (addr != INADDR_NONE) return ntohl(addr);
    he = gethostbyname(name);
    if (he) {
	if (he->h_addrtype == AF_INET) return ntohl(*(uint32_t *) he->h_addr);
	yyerror("invalid IPv4 address");
    }
    fflush(stdout);
    herror(name);
    exit(1);
}


U128 ipv6_host(const char *name,int allow_dns)
{
    struct addrinfo hints,*res;
    unsigned char *a;
    U128 r;
    int error;

    memset(&hints,0,sizeof(hints));
    hints.ai_flags = allow_dns ? 0 : AI_NUMERICHOST;
    hints.ai_family = PF_INET6;
    hints.ai_socktype = 0;
    hints.ai_protocol = 0;
    error = getaddrinfo(name,NULL,&hints,&res);
    if (error) yyerror(gai_strerror(error));
    a = ((struct sockaddr_in6 *) res->ai_addr)->sin6_addr.s6_addr;
    r.v[3] = (a[0] << 24) | (a[1] << 16) | (a[2] << 8) | a[3];
    r.v[2] = (a[4] << 24) | (a[5] << 16) | (a[6] << 8) | a[7];
    r.v[1] = (a[8] << 24) | (a[9] << 16) | (a[10] << 8) | a[11];
    r.v[0] = (a[12] << 24) | (a[13] << 16) | (a[14] << 8) | a[15];
    freeaddrinfo(res);
    return r;
}
